@@ -67,10 +67,11 @@ class PostgreSQL(base.Plugin):
         self.add_backend_stats(cursor)
         self.add_bgwriter_stats(cursor)
         self.add_database_stats(cursor)
-        self.add_index_stats(cursor)
         self.add_lock_stats(cursor)
-        self.add_statio_stats(cursor)
-        self.add_table_stats(cursor)
+        if self.config.get('relation_stats', True):
+            self.add_index_stats(cursor)
+            self.add_statio_stats(cursor)
+            self.add_table_stats(cursor)
         self.add_transaction_stats(cursor)
 
         # add_wal_metrics needs superuser to get directory listings
@@ -230,22 +231,24 @@ class PostgreSQL(base.Plugin):
         :rtype: psycopg2.connection
 
         """
-        conn = psycopg2.connect(self.dsn)
+        conn = psycopg2.connect(**self.connection_arguments)
         conn.set_isolation_level(extensions.ISOLATION_LEVEL_AUTOCOMMIT)
         return conn
 
     @property
-    def dsn(self):
-        """Create a DSN to connect to
-
-        :return str: The DSN to connect
+    def connection_arguments(self):
+        """Create connection parameter dictionary for psycopg2.connect
 
+        :return dict: The dictionary to be passed to psycopg2.connect via double-splat
         """
-        dsn = "host='%(host)s' port=%(port)i dbname='%(dbname)s' " \
-              "user='%(user)s'" % self.config
-        if self.config.get('password'):
-            dsn += " password='%s'" % self.config['password']
-        return dsn
+        filtered_args = ["name","superuser"]
+        args = {}
+        for key in set(self.config) - set(filtered_args):
+            if key == 'dbname':
+                args['database'] = self.config[key]
+            else:
+                args[key] = self.config[key]
+        return args
 
     def poll(self):
         self.initialize()
